Struct granular_id::GranularId

source ·
pub struct GranularId<T> { /* private fields */ }
The data type GranularId represents any ID which can have an arbitrary granularity, meaning that there is indefinitely many GranularIds in-between any two GranularIds. It is best used with unsized integer types such as u8, u16, u32, u64 and usize, but may be used with other data types implementing the required num_traits bounds.

An ID basically consists of multiple components

Implementations

source

impl<T> GranularId<T>where T: SequentialId,

source

pub fn all_siblings(&self) -> SequentialIds<T> where T: LowerBounded,

Gets all the siblings of this GranularId, ordered from the smallest sibling to the largest sibling. This is all the GranularIds which starts the same as this GranularId but has its last component changed.

use granular_id::GranularId;
let id = GranularId::from(vec![1u8, 2, 2]); // id 1.2.2
let mut all_siblings = id.all_siblings();
assert_eq!(all_siblings.next().unwrap(), vec![1, 2, 0].into()); // id 1.2.0
assert_eq!(all_siblings.next().unwrap(), vec![1, 2, 1].into()); // id 1.2.1
assert_eq!(all_siblings.next().unwrap(), vec![1, 2, 2].into()); // id 1.2.2
assert_eq!(all_siblings.next().unwrap(), vec![1, 2, 3].into()); // id 1.2.3
source

pub fn next_siblings(&self) -> SequentialIds<T> where T: CheckedAdd,

Gets all the next siblings of this GranularId, ordered from the smallest sibling larger than this ID to the largest sibling. This is all the GranularIds which starts the same as this GranularId but has its last component changed to any larger value.

use granular_id::GranularId;
let id = GranularId::from(vec![1u8, 2, 2]); // id 1.2.2
let mut next_siblings = id.next_siblings();
assert_eq!(next_siblings.next().unwrap(), vec![1, 2, 3].into()); // id 1.2.3
assert_eq!(next_siblings.next().unwrap(), vec![1, 2, 4].into()); // id 1.2.4
assert_eq!(next_siblings.next().unwrap(), vec![1, 2, 5].into()); // id 1.2.5
assert_eq!(next_siblings.next().unwrap(), vec![1, 2, 6].into()); // id 1.2.6
source

pub fn children(&self) -> SequentialIds<T> where T: LowerBounded,

Gets all direct children of this GranularId, ordered from the smallest child (which is larger than this ID, but smallest out of all children) to the largest child. This is all the GranularIds which starts with this GranularId but has one additional component.

use granular_id::GranularId;
let id = GranularId::from(vec![1u8, 2]); // id 1.2
let mut children = id.children();
assert_eq!(children.next().unwrap(), vec![1, 2, 0].into()); // id 1.2.0
assert_eq!(children.next().unwrap(), vec![1, 2, 1].into()); // id 1.2.1
assert_eq!(children.next().unwrap(), vec![1, 2, 2].into()); // id 1.2.2
assert_eq!(children.next().unwrap(), vec![1, 2, 3].into()); // id 1.2.3
source

impl<T> GranularId<T>where T: BackSequentialId,

source

pub fn previous_siblings(&self) -> BackSequentialIds<T> where T: Bounded + One + Clone + CheckedSub,

Gets all the previous siblings of this GranularId, ordered from the largest sibling smaller than this ID to the smallest sibling. This is all the GranularIds which starts the same as this GranularId but has its last component changed to any smaller value.

use granular_id::GranularId;
let id = GranularId::from(vec![1u8, 2, 3]); // id 1.2.3
let mut previous_siblings = id.previous_siblings();
assert_eq!(previous_siblings.next().unwrap(), vec![1, 2, 2].into()); // id 1.2.2
assert_eq!(previous_siblings.next().unwrap(), vec![1, 2, 1].into()); // id 1.2.1
assert_eq!(previous_siblings.next().unwrap(), vec![1, 2, 0].into()); // id 1.2.0
assert_eq!(previous_siblings.next(), None); // No more smaller siblings
source

impl<T> GranularId<T>

source

pub fn is_max_value(&self) -> boolwhere T: UpperBounded + PartialEq,

Checks whether this GranularId is the maximum value of this type. If it is, no other larger GranularIds exist of the same type.

use granular_id::GranularId;
let not_max: GranularId<u8> = vec![254].into();
let max: GranularId<u8> = vec![255].into();
assert!(!not_max.is_max_value());
assert!(max.is_max_value());
source

pub fn granularity(&self) -> usize

Gets the granularity of this GranularId, which is the number of components it contains, or the “precision” it has


use granular_id::GranularId;
let id: GranularId<u8> = vec![1, 2, 3, 4].into(); // id 1.2.3.4, granularity 4
assert_eq!(id.granularity(), 4);
// Its children should have granularity 5:
assert_eq!(id.children().next().unwrap().granularity(), 5);
source

pub fn is_root(&self) -> bool

Checks whether this GranularId is the root, which means not having any components.

use granular_id::GranularId;
let not_root: GranularId<u8> = vec![254].into();
let root: GranularId<u8> = vec![].into();
assert!(!not_root.is_root());
assert!(root.is_root());
source

pub fn into_parent(self) -> Self

Turns this GranularId into its parent, that is, the same GranularId with the last component removed. If this GranularId is the root (has no components), this function returns the root. For a mutating version, see GranularId::pop, and for a copying version, see GranularId::parent

use granular_id::GranularId;
let id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
assert_eq!(id.into_parent(), vec![1, 2].into()); // id 1.2
source

pub fn parent(&self) -> Selfwhere T: Clone,

Gets the parent of this GranularId, that is, the same GranularId with the last component removed.

use granular_id::GranularId;
let id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
assert_eq!(id.parent(), vec![1, 2].into()); // id 1.2
source

pub fn pop(&mut self)

Removes the last component of this GranularId. To get the parent of any GranularId without mutating it, see GranularId::parent

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
id.pop();
assert_eq!(id, vec![1, 2].into()); // id 1.2
source

pub fn push(&mut self, component: T)where T: UpperBounded + PartialEq,

Pushes a new component to the GranularId, adding it as a new last component. Since the maximum bound for a GranularId of type T is [T::max], if any component is pushed to such an ID, the call doesn’t do anything.

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
id.push(4);
assert_eq!(id, vec![1, 2, 3, 4].into()); // id 1.2.3.4
source

pub fn pushing(&self, component: T) -> Selfwhere T: UpperBounded + PartialEq + Clone,

Makes a new GranularId, adding the given component as its new last component. Since the maximum bound for a GranularId of type T is [T::max], if any component is pushed to such an ID, the call returns a plain copy of the original GranularId.

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
assert_eq!(id.pushing(4), vec![1, 2, 3, 4].into()); // id 1.2.3.4
source

pub fn append(&mut self, other: GranularId<T>)where T: UpperBounded + PartialEq,

Appends all components from another GranularId to this GranularId, adding them as a new last components. Since the maximum bound for a GranularId of type T is [T::max], if any components is appended to such an ID, the call doesn’t do anything.

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
let id_2 = vec![4, 5, 6].into(); // id 4.5.6
id.append(id_2);
assert_eq!(id, vec![1, 2, 3, 4, 5, 6].into()); // id 1.2.3.4.5.6
source

pub fn appending(&self, other: GranularId<T>) -> GranularId<T>where T: UpperBounded + PartialEq + Clone,

Makes a new GranularId, appending all components from another GranularId to that GranularId, adding them as a new last components. Since the maximum bound for a GranularId of type T is [T::max], if any components is appended to such an ID, the call returns a copy of the original GranularId.

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
let id_2 = vec![4, 5, 6].into(); // id 4.5.6
assert_eq!(id.appending(id_2), vec![1, 2, 3, 4, 5, 6].into()); // id 1.2.3.4.5.6
source

pub fn append_vec(&mut self, other: Vec<T>)where T: UpperBounded + PartialEq,

Appends all components from a Vec to this GranularId, adding them as a new last components. Since the maximum bound for a GranularId of type T is [T::max], if any components is appended to such an ID, the call doesn’t do anything.

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
id.append_vec(vec![4, 5, 6]);
assert_eq!(id, vec![1, 2, 3, 4, 5, 6].into()); // id 1.2.3.4.5.6
source

pub fn appending_vec(&self, other: Vec<T>) -> GranularId<T>where T: UpperBounded + PartialEq + Clone,

Makes a new GranularId, appending all components from a Vec to that GranularId, adding them as a new last components. Since the maximum bound for a GranularId of type T is [T::max], if any components is appended to such an ID, the call returns a copy of the original GranularId.

use granular_id::GranularId;
let mut id: GranularId<u8> = vec![1, 2, 3].into(); // id 1.2.3
assert_eq!(id.appending_vec(vec![4, 5, 6]), vec![1, 2, 3, 4, 5, 6].into()); // id 1.2.3.4.5.6
source

pub fn root() -> Self

Gets the root GranularId of any type, that is, a GranularId without any components. That GranularId is the smallest one of that type.

use granular_id::GranularId;
let id: GranularId<u8> = GranularId::root(); // id <root>
assert_eq!(Into::<Vec<u8>>::into(id), vec![]);